
# Load the SOAP service support framework
package require SOAP::Domain

global MIDAS_SOAPService_Verbose
set MIDAS_SOAPService_Verbose 0

# Use namespaces to isolate your methods
namespace eval urn:CommandServer {


    proc Command {p} {
        global MIDAS_SOAPService_Verbose
        if {$MIDAS_SOAPService_Verbose} {puts "called method Command with parameter $p"}
        set z [catch {eval $p} m]
        set status "$m"
        return [rpcvar::rpcvar string $status]
    }


    # We have to publish the public methods...

     SOAP::export Command
}

# register this service with tclhttpd
puts [SOAP::Domain::register  -prefix  /CommandServer -namespace urn:CommandServer -uri urn:CommandServer]


global MIDAS_SOAPDomainHandler_Debug
set MIDAS_SOAPDomainHandler_Debug 0

proc ::SOAP::Domain::MIDAS_SOAPDomainHandler_/CommandServer {optsname sock args} {
    global MIDAS_SOAPDomainHandler_Debug
    variable debug
    upvar \#0 Httpd$sock data
    upvar \#0 $optsname options

  if {$MIDAS_SOAPDomainHandler_Debug} {
      puts "::SOAP::Domain::MIDAS_SOAPDomainHandler_/CommandServer called with $optsname || $sock || $args"
      foreach member [array names options] {puts "options($member) =  $options($member)"}
      foreach member [array names data] {puts "data($member) =  $data($member)"}
      puts ""
  }

    # if suffix is {} then it fails to make it through the various evals.
    set suffix [lindex $args 0]
    
    # check this is an XML post
    set failed [catch {set type $data(mime,content-type)} msg]
    if { $failed } {
        set msg "Invalid SOAP request: not XML data"
        log::log debug $msg
        Httpd_ReturnData $sock text/xml [SOAP::fault SOAP-ENV:Client $msg] 500
        return $failed
    }
    
    # make sure we were sent some XML
    set failed [catch {set query $data(query)} msg]
    if { $failed } {
        set msg "Invalid SOAP request: no data sent"
        log::log debug $msg
        Httpd_ReturnData $sock text/xml [SOAP::fault SOAP-ENV:Client $msg] 500
        return $failed
    }

    # Check that we have a properly registered domain
    if { ! [info exists options] } {
        set msg "Internal server error: domain improperly registered"
        log::log debug $msg
        Httpd_ReturnData $sock text/xml [SOAP::fault SOAP-ENV:Server $msg] 500
        return 1
    }   

   set argValues {}

   set frags [split $data(query) >]
   set fragNum [llength $frags]

   for {set i 0} {$i < $fragNum} {incr i} {
       set frag [lindex $frags $i]
       if {[string compare -length 14 $frag <SOAP-ENV:Body] == 0} {
            incr i
            set frag [lindex $frags $i]
            set methodName [string range [lindex [split $frag " "] 0] 4 end]
            incr i
            for {} {$i < $fragNum} {incr i} {
                set frag [lindex $frags $i]
                if {[string compare -length 3 $frag </ns] == 0} {break}
                    incr i
                    set frag [lindex $frags $i]
                    lappend argValues [lindex [split $frag <] 0]
            }
       }
   }

       # evaluate the method
       set z [catch {interp eval "" ::namespace inscope $options(-namespace) $methodName $argValues} msg]

    if {$z == 0}  {

       set xml "<?xml version=\"1.0\"?>"
       append xml "<SOAP-ENV:Envelope xmlns:xsi=\"http://www.w3.org/1999/XMLSchema-instance\" "
       append xml "xmlns:SOAP-ENV=\"http://schemas.xmlsoap.org/soap/envelope/\" "
       append xml "xmlns:SOAP-ENC=\"http://schemas.xmlsoap.org/soap/encoding/\" "
       append xml "xmlns:xsd=\"http://www.w3.org/1999/XMLSchema\">"

       append xml "<SOAP-ENV:Body>"

       append xml "<ns:[set methodName]Response xmlns:ns=\"\" SOAP-ENV:encodingStyle=\"http://schemas.xmlsoap.org/soap/encoding/\">"
       append xml "<return xsi:type=\"xsd:[lindex $msg 4]\">[lindex $msg 5]</return>"
       append xml "</ns:[set methodName]Response>"

       append xml "</SOAP-ENV:Body>"

       append xml "</SOAP-ENV:Envelope>"

       set xml [encoding convertto utf-8 $xml]          ;# make it UTF-8

    Httpd_ReturnData $sock text/xml $xml [expr {$failed ? 500 : 200}]

       set failed 0

    } else {
 
       set failed 1
    }

    return $failed
}

# -------------------------------------------------------------------------
